package tutorial;


/**
 * This is the client portion of the beginner's tutorial on how to use declarative
 * RMI annotations.  The client passes some instances of class A (a POJO which is
 * serializable but not remote) to a server method, foo.  The foo method specifies
 * different passing semantics for each of the three parameters.  Thus, instances of
 * the same run time type (A) are passed by different semantics.
 * 
 * Note that if this were to be done in regular RMI, a wrapper class around A
 * would need to be created and the remote reference parameter to foo would
 * need to be changed to A's remote interface.  Even for this small example,
 * it's clear that a declarative approach is more desirable.
 *
 * @author Wesley Tansey
 * @date Sep 24, 2007
 */
public class Client {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length != 1) {
			System.err.println("Usage: java Client url");
			return;
		}
		
		ServerInterface server = null;
		
		try {
			server = (ServerInterface) java.rmi.Naming.lookup(args[0]);
		} catch (java.rmi.NotBoundException e) {
			e.printStackTrace();
			return;
		} catch (java.net.MalformedURLException e1) {
			e1.printStackTrace();
			return;
		} catch (java.rmi.RemoteException e2) {
			e2.printStackTrace();
			return;
		}
		
		try {
			
			//We first create 3 objects to be passed to the server method.
			//Each will be passed by a different semantics.
			A byCopy = new A();
			A byRemoteReference = new A();
			A byCopyRestore = new A();
			
			//Print off the initial values, before the objects have been passed
			//to the server. The values should all be 0.
			System.out.println("byCopy: initial value=" + byCopy.getValue());
			System.out.println("byRemoteReference: initial value=" + byRemoteReference.getValue());
			System.out.println("byCopyRestore: initial value=" + byCopyRestore.getValue());
			System.out.println();
			
			//Next, we call the server method, passing it our three objects.
			server.foo(byCopy, byRemoteReference, byCopyRestore);
			
			//Print off the resulting values, after the objects have been passed
			//to the server. The byCopy object's value should still be 0, as it
			//was passed by copy.  However, the byRemoteReference and byCopyRestore
			//objects should have been updated on the server, and their new values
			//should be 1.
			System.out.println("byCopy: new value=" + byCopy.getValue());
			System.out.println("byRemoteReference: new value=" + byRemoteReference.getValue());
			System.out.println("byCopyRestore: new value=" + byCopyRestore.getValue());
			System.out.println();
			
		} catch (java.rmi.RemoteException e) {
			e.printStackTrace();
		}
	}

}
